/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.core;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;

import benten.core.BentenConstants;
import benten.core.io.Files;
import benten.core.io.TmxFileFilter;
import benten.core.model.BentenTmx;
import benten.core.model.BentenXliff;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import benten.twa.tmx.core.valueobject.BentenExtractTmxFragmentProcessInput;
import benten.twa.tmx.messages.BentenExtractTmxFragmentMessages;
import blanco.tmx.valueobject.BlancoTmxTu;
import blanco.tmx.valueobject.BlancoTmxTuv;
import blanco.xliff.BlancoXliffUtil;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * TMX フラグメントの抽出
 *
 * <pre>
 * XLIFF フラグメントから、TMX フラグメントを抽出します。
 *   1.  XLIFF から翻訳単位を抽出して TMX を作成します。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳メモリー断片抽出・マージ機能: TMXフラグメント抽出機能」に対応します。
 */
public class BentenExtractTmxFragmentProcessImpl extends AbstractTraverseDir implements BentenExtractTmxFragmentProcess {
	/**
	 * TMXフラグメント抽出機能のためのメッセージ。
	 */
	protected static final BentenExtractTmxFragmentMessages fMsg = new BentenExtractTmxFragmentMessages();

	/**
	 * この処理の入力オブジェクト。
	 */
	protected BentenExtractTmxFragmentProcessInput fInput;

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	private final String nowDate = new SimpleDateFormat("yyyyMMdd").format(new Date()); //$NON-NLS-1$

	private BentenTmx fOutTmx;

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenExtractTmxFragmentProcessInput input) throws IOException, IllegalArgumentException {
		if (input == null) {
			throw new IllegalArgumentException("BentenExtractTmxFragmentProcessImpl#execute: argument 'input' is null."); //$NON-NLS-1$
		}
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		final File fileTmx = createFile(new File(input.getTranstargetid()), new File(input.getTmxdir()), input
				.getIncludedate());
		fOutTmx = BentenTmx.newInstance(fileTmx);

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		processDir(new File(input.getXliffdir()));

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		Collections.sort(fOutTmx.getTuList(), new BentenTmxTuComparator());
		BentenTmxTuUtil.removeDuplicatedTuv(fOutTmx.getTuList());

		if (progress(fMsg.getCoreP004(fileTmx.getName()))) {
			return 6;
		}

		fOutTmx.save();

		if (progress(fMsg.getCoreP005())) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	/**
	 * ファイルの作成。
	 * @param targetFile
	 * @param toDir
	 * @param includeDate
	 * @return
	 */
	protected File createFile(final File targetFile, final File toDir, final boolean includeDate) {
		final StringBuilder name = new StringBuilder();
		name.append(Files.baseName(targetFile));
		if (includeDate) {
			name.append("-" + nowDate); //$NON-NLS-1$
		}
		return TmxFileFilter.appendExtension(new File(toDir, name.toString()));
	}

	@Override
	protected boolean canProcess(File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	protected void processFile(final File file, final String baseDir) throws IOException {
		if (file == null) {
			throw new IllegalArgumentException(
					"BentenExtractTmxFragmentProcessImpl#processFile: argument 'file' is null."); //$NON-NLS-1$
		}
		if (baseDir == null) {
			throw new IllegalArgumentException(
					"BentenExtractTmxFragmentProcessImpl#processFile: argument 'baseDir' is null."); //$NON-NLS-1$
		}

		if (fInput == null) {
			throw new IllegalArgumentException(
					"BentenExtractTmxFragmentProcessImpl#processFile: 'fInput' is null. Call execute or setInput before calling this method."); //$NON-NLS-1$
		}

		if (progress(fMsg.getCoreP011(file.getName()))) {
			return;
		}

		final BentenXliff inXliff = BentenXliff.loadInstance(file);

		int transUnitCount = 0;
		int transUnitTotalCount = 0;
		for (BlancoXliffFile inFile : inXliff.getFileList()) {
			transUnitTotalCount += inFile.getBody().getTransUnitList().size();
			for (BlancoXliffTransUnit inUnit : inFile.getBody().getTransUnitList()) {
				if (progress(fMsg.getCoreP101(file.getName(), BigDecimal.valueOf(++transUnitCount), BigDecimal
						.valueOf(transUnitTotalCount)))) {
					break;
				}

				if (inUnit.getTranslate() == false) {
					continue;
				}
				if (inUnit.getSource() == null || inUnit.getSource().length() == 0) {
					continue;
				}
				if (inUnit.getTarget() == null || inUnit.getTarget().getTarget() == null
						|| inUnit.getTarget().getTarget().length() == 0) {
					continue;
				}

				// TMX反映除外のチェック。
				if (BlancoXliffUtil.isXTmOmit(inUnit)) {
					continue;
				}

				// state のチェック
				if (fInput.getExtractbystate() != null) {
					if (fInput.getExtractbystate().trim().length() == 0) {
						// state 無しを探したい。
						if (inUnit.getTarget().getState() == null || inUnit.getTarget().getState().trim().length() == 0) {
							// これが出力対象。
						} else {
							// 出力対象外。
							continue;
						}
					} else {
						// 通常の state 指定。
						if (fInput.getExtractbystate().equals(inUnit.getTarget().getState())) {
							// state が一致します。処理対象です。
						} else {
							// state が異なるので処理対象外とします。
							continue;
						}
					}
				}

				fResultInfo.setUnitCount(fResultInfo.getUnitCount() + 1);

				final BlancoTmxTu tu = new BlancoTmxTu();
				fOutTmx.getTuList().add(tu);
				getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
				{
					BlancoTmxTuv tuv = new BlancoTmxTuv();
					tu.getTuvList().add(tuv);
					tuv.setLang(inFile.getSourceLanguage());
					tuv.setSeg(inUnit.getSource());
				}
				{
					BlancoTmxTuv tuv = new BlancoTmxTuv();
					tu.getTuvList().add(tuv);
					tuv.setLang(inFile.getTargetLanguage());
					tuv.setSeg(inUnit.getTarget().getTarget());
				}
			}
		}
	}
}
