/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.core;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import benten.core.io.Files;
import benten.core.io.TmxFileFilter;
import benten.twa.process.BentenProcessResultInfo;
import benten.twa.tmx.core.valueobject.BentenMergeTmxProcessInput;
import benten.twa.tmx.messages.BentenMergeTmxMessages;
import benten.twa.tmx.status.BentenMergeTmxStatus;
import blanco.tmx.BlancoTmxParser;
import blanco.tmx.BlancoTmxSerializer;
import blanco.tmx.BlancoTmxUtil;
import blanco.tmx.valueobject.BlancoTmx;
import blanco.tmx.valueobject.BlancoTmxTu;

/**
 * TMX フラグメントのマージ
 *
 * <pre>
 * TMX フラグメントを TMX にマージします。
 *   1.  TMX の内容を、別の TMX にマージします。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳メモリー断片抽出・マージ機能: TMXフラグメント・マージ機能」に対応します。
 */
public class BentenMergeTmxProcessImpl implements BentenMergeTmxProcess {
	/**
	 * TMXマージ機能のためのメッセージ。
	 */
	protected static final BentenMergeTmxMessages fMsg = new BentenMergeTmxMessages();

	/**
	 * この処理の入力オブジェクト。
	 */
	protected BentenMergeTmxProcessInput fInput = null;

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenMergeTmxProcessInput input) throws IOException, IllegalArgumentException {
		if (input == null) {
			throw new IllegalArgumentException("BentenMergeTmxProcessImpl#execute: argument 'input' is null."); //$NON-NLS-1$
		}
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		if (progress(fMsg.getCoreP011(input.getSourcefile()))) {
			return 6;
		}
		final BlancoTmx tmxIn = new BlancoTmxParser().parse(new File(input.getSourcefile()));

		if (progress(fMsg.getCoreP012(input.getTargetfile()))) {
			return 6;
		}
		final BlancoTmx tmxOut = new BlancoTmxParser().parse(new File(input.getTargetfile()));
		final int originalSize = tmxOut.getBody().getTuList().size();

		if (progress(fMsg.getCoreP013())) {
			return 6;
		}
		final Map<String, String> mapTmxOut = new HashMap<String, String>(16384);
		for (BlancoTmxTu tu : tmxOut.getBody().getTuList()) {
			final String en = BlancoTmxUtil.getTuvByLang(tu, "EN").getSeg(); //$NON-NLS-1$
			final String ja = BlancoTmxUtil.getTuvByLang(tu, "JA").getSeg(); //$NON-NLS-1$
			mapTmxOut.put("en:[" + en + "], ja:[" + ja + "]", "exist"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}

		if (progress(fMsg.getCoreP014())) {
			return 6;
		}
		for (BlancoTmxTu tu : tmxIn.getBody().getTuList()) {
			final String en = BlancoTmxUtil.getTuvByLang(tu, "EN").getSeg(); //$NON-NLS-1$
			final String ja = BlancoTmxUtil.getTuvByLang(tu, "JA").getSeg(); //$NON-NLS-1$
			if (mapTmxOut.get("en:[" + en + "], ja:[" + ja + "]") != null) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				// すでに一致する登録が存在。
				// TODO 現時点では、一致する登録は音も無く無視しています。
			} else {
				tmxOut.getBody().getTuList().add(tu);
			}
		}

		if (progress(fMsg.getCoreP015())) {
			return 6;
		}

		Collections.sort(tmxOut.getBody().getTuList(), new BentenTmxTuComparator());
		BentenTmxTuUtil.removeDuplicatedTuv(tmxOut.getBody().getTuList());
		final int afterSize = tmxOut.getBody().getTuList().size();

		fResultInfo.setUnitCount(fResultInfo.getUnitCount() + Math.max(afterSize - originalSize, 0));

		if (progress(fMsg.getCoreP016(input.getTargetfile()))) {
			return 6;
		}

		new BlancoTmxSerializer().serialize(tmxOut, new File(input.getTargetfile()));
		fResultInfo.setSuccessCount(fResultInfo.getSuccessCount() + 1);

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	/**
	 * ファイルの作成。
	 * TODO Eclipse UI 由来のクラスが混じっています。
	 * @param status
	 * @return
	 */
	protected File createFile(BentenMergeTmxStatus status) {
		StringBuilder name = new StringBuilder();
		name.append(Files.baseName(status.getTargetFile()));
		return TmxFileFilter.appendExtension(new File(status.getToFile(), name.toString()));
	}
}
